/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    d_naout.c
 *
 * Purpose: MX network analog output device driver.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "mx_util.h"
#include "mx_drivr.h"
#include "mx_net.h"
#include "mx_aout.h"
#include "d_naout.h"

MX_RECORD_FUNCTION_LIST mxd_network_aoutput_record_function_list = {
	mxd_network_aoutput_initialize_type,
	NULL,
	NULL,
	mxd_network_aoutput_create_record_structures,
	mxd_network_aoutput_finish_record_initialization,
	mxd_network_aoutput_delete_record,
	NULL,
	NULL,
	mxd_network_aoutput_read_parms_from_hardware,
	mxd_network_aoutput_write_parms_to_hardware,
	mxd_network_aoutput_open,
	mxd_network_aoutput_close
};

MX_ANALOG_OUTPUT_FUNCTION_LIST mxd_network_aoutput_analog_output_function_list = {
	mxd_network_aoutput_read,
	mxd_network_aoutput_write
};

MX_RECORD_FIELD_DEFAULTS mxd_network_aoutput_record_field_defaults[] = {
	MX_RECORD_STANDARD_FIELDS,
	MX_ANALOG_OUTPUT_STANDARD_FIELDS,
	MXD_NETWORK_AOUTPUT_STANDARD_FIELDS
};

long mxd_network_aoutput_num_record_fields
		= sizeof( mxd_network_aoutput_record_field_defaults )
			/ sizeof( mxd_network_aoutput_record_field_defaults[0] );

MX_RECORD_FIELD_DEFAULTS *mxd_network_aoutput_rfield_def_ptr
			= &mxd_network_aoutput_record_field_defaults[0];

/* ===== */

MX_EXPORT mx_status_type
mxd_network_aoutput_initialize_type( long type )
{
	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_network_aoutput_create_record_structures( MX_RECORD *record )
{
        const char fname[] = "mxd_network_aoutput_create_record_structures()";

        MX_ANALOG_OUTPUT *analog_output;
        MX_NETWORK_AOUTPUT *network_aoutput;

        /* Allocate memory for the necessary structures. */

        analog_output = (MX_ANALOG_OUTPUT *) malloc(sizeof(MX_ANALOG_OUTPUT));

        if ( analog_output == (MX_ANALOG_OUTPUT *) NULL ) {
                return mx_error( MXE_OUT_OF_MEMORY, fname,
                "Can't allocate memory for MX_ANALOG_OUTPUT structure." );
        }

        network_aoutput = (MX_NETWORK_AOUTPUT *)
				malloc( sizeof(MX_NETWORK_AOUTPUT) );

        if ( network_aoutput == (MX_NETWORK_AOUTPUT *) NULL ) {
                return mx_error( MXE_OUT_OF_MEMORY, fname,
                "Can't allocate memory for MX_NETWORK_AOUTPUT structure." );
        }

        /* Now set up the necessary pointers. */

        record->record_class_struct = analog_output;
        record->record_type_struct = network_aoutput;
        record->class_specific_function_list
			= &mxd_network_aoutput_analog_output_function_list;

        analog_output->record = record;

        return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_network_aoutput_finish_record_initialization( MX_RECORD *record )
{
	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_network_aoutput_delete_record( MX_RECORD *record )
{
        if ( record == NULL ) {
                return MX_SUCCESSFUL_RESULT;
        }
        if ( record->record_type_struct != NULL ) {
                free( record->record_type_struct );

                record->record_type_struct = NULL;
        }
        if ( record->record_class_struct != NULL ) {
                free( record->record_class_struct );

                record->record_class_struct = NULL;
        }
        return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_network_aoutput_read_parms_from_hardware( MX_RECORD *record )
{
	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_network_aoutput_write_parms_to_hardware( MX_RECORD *record )
{
	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_network_aoutput_open( MX_RECORD *record )
{
	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_network_aoutput_close( MX_RECORD *record )
{
	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxd_network_aoutput_read( MX_ANALOG_OUTPUT *aoutput )
{
	const char fname[] = "mxd_network_aoutput_read()";

	MX_NETWORK_AOUTPUT *network_aoutput;
	char rfname[ MXU_RECORD_FIELD_NAME_LENGTH ];
	long raw_value;
	mx_status_type status;

	network_aoutput = (MX_NETWORK_AOUTPUT *)
				aoutput->record->record_type_struct;

	if ( network_aoutput == NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"MX_NETWORK_AOUTPUT pointer for analog output '%s' is NULL",
			aoutput->record->name );
	}

	sprintf( rfname, "%s.raw_value", network_aoutput->remote_record_name );

	status = mx_get( network_aoutput->server_record,
				rfname, MXFT_LONG, &raw_value );

	aoutput->raw_value = raw_value;

	return status;
}

MX_EXPORT mx_status_type
mxd_network_aoutput_write( MX_ANALOG_OUTPUT *aoutput )
{
	const char fname[] = "mxd_network_aoutput_read()";

	MX_NETWORK_AOUTPUT *network_aoutput;
	char rfname[ MXU_RECORD_FIELD_NAME_LENGTH ];
	long raw_value;
	mx_status_type status;

	network_aoutput = (MX_NETWORK_AOUTPUT *)
				aoutput->record->record_type_struct;

	if ( network_aoutput == NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"MX_NETWORK_AOUTPUT pointer for analog output '%s' is NULL",
			aoutput->record->name );
	}

	raw_value = aoutput->raw_value;

	sprintf( rfname, "%s.raw_value", network_aoutput->remote_record_name );

	status = mx_put( network_aoutput->server_record,
				rfname, MXFT_LONG, &raw_value );

	return status;
}

