/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    d_pdi45.h
 *
 * Purpose: Header file for MX drivers for the PDI Model 45 DAQ interface.
 */

#ifndef __D_PDI45_H__
#define __D_PDI45_H__

#include "mx_ain.h"
#include "mx_aout.h"
#include "mx_din.h"
#include "mx_dout.h"
#include "mx_scalr.h"
#include "mx_timer.h"

#include "pdi45.h"

/* ============ Generic functions ============ */

MX_API mx_status_type mxd_pdi45_initialize_type( long type );
MX_API mx_status_type mxd_pdi45_read_record_description( char *buffer,
			MX_RECORD *record, MX_RECORD *record_list_head );
MX_API mx_status_type mxd_pdi45_write_record_description(
			MX_RECORD *record, char *buffer );
MX_API mx_status_type mxd_pdi45_delete_record( MX_RECORD *record );
MX_API mx_status_type mxd_pdi45_print_structure( FILE *file,
							MX_RECORD *record );
MX_API mx_status_type mxd_pdi45_print_summary( FILE *file,
							MX_RECORD *record );
MX_API mx_status_type mxd_pdi45_read_parms_from_hardware( MX_RECORD *record );
MX_API mx_status_type mxd_pdi45_write_parms_to_hardware( MX_RECORD *record );
MX_API mx_status_type mxd_pdi45_open( MX_RECORD *record );
MX_API mx_status_type mxd_pdi45_close( MX_RECORD *record );

extern MX_RECORD_FUNCTION_LIST mxd_pdi45_record_function_list;

/* ============ Analog input channels ============ */

typedef struct {
	MX_RECORD *pdi45_record;
	MX_PDI45 *pdi45;
	int channel;
} MX_PDI45_ADC;

MX_API mx_status_type mxd_pdi45adc_read( MX_ANALOG_INPUT *adc );

extern MX_ANALOG_INPUT_FUNCTION_LIST mxd_pdi45adc_analog_input_function_list;

/* ============ Analog output channels ============ */

typedef struct {
	MX_RECORD *pdi45_record;
	MX_PDI45 *pdi45;
	int channel;
} MX_PDI45_DAC;

MX_API mx_status_type mxd_pdi45dac_read( MX_ANALOG_OUTPUT *dac );
MX_API mx_status_type mxd_pdi45dac_write( MX_ANALOG_OUTPUT *dac );

extern MX_ANALOG_OUTPUT_FUNCTION_LIST mxd_pdi45dac_analog_output_function_list;

/* ============ Digital input channels ============ */

typedef struct {
	MX_RECORD *pdi45_record;
	MX_PDI45 *pdi45;
	int channel;
} MX_PDI45_DIN;

MX_API mx_status_type mxd_pdi45din_read( MX_DIGITAL_INPUT *dinput );

extern MX_DIGITAL_INPUT_FUNCTION_LIST mxd_pdi45din_digital_input_function_list;

/* ============ Digital output channels ============ */

typedef struct {
	MX_RECORD *pdi45_record;
	MX_PDI45 *pdi45;
	int channel;
} MX_PDI45_DOUT;

MX_API mx_status_type mxd_pdi45dout_read( MX_DIGITAL_OUTPUT *doutput );
MX_API mx_status_type mxd_pdi45dout_write( MX_DIGITAL_OUTPUT *doutput );

extern MX_DIGITAL_OUTPUT_FUNCTION_LIST
				mxd_pdi45dout_digital_output_function_list;

/* ============ Scaler channels ============ */

typedef struct {
	MX_RECORD *pdi45_record;
	MX_PDI45 *pdi45;
	int channel;
} MX_PDI45_SCALER;

MX_API mx_status_type mxd_pdi45scaler_clear( MX_SCALER *scaler );
MX_API mx_status_type mxd_pdi45scaler_overflow_set( MX_SCALER *scaler );
MX_API mx_status_type mxd_pdi45scaler_read( MX_SCALER *scaler );
MX_API mx_status_type mxd_pdi45scaler_read_and_clear( MX_SCALER *scaler );
MX_API mx_status_type mxd_pdi45scaler_write( MX_SCALER *scaler );

extern MX_SCALER_FUNCTION_LIST mxd_pdi45scaler_scaler_function_list;

/* ============ Timer channels ============ */

typedef struct {
	MX_RECORD *pdi45_record;
	MX_PDI45 *pdi45;
	int channel;
} MX_PDI45_TIMER;

MX_API mx_status_type mxd_pdi45timer_is_busy( MX_TIMER *timer, int *busy );
MX_API mx_status_type mxd_pdi45timer_start( 
					MX_TIMER *timer, double seconds );
MX_API mx_status_type mxd_pdi45timer_stop( 
					MX_TIMER *timer, double *seconds_left);
MX_API mx_status_type mxd_pdi45timer_get_current_time(
					MX_TIMER *timer, double *seconds );
MX_API mx_status_type mxd_pdi45timer_set_current_count( MX_TIMER *timer,
							unsigned long count );
MX_API mx_status_type mxd_pdi45timer_get_current_count( MX_TIMER *timer,
							unsigned long *count );

extern MX_TIMER_FUNCTION_LIST mxd_pdi45timer_timer_function_list;

#endif /* __D_PDI45_H__ */
