/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    d_pmacdp.h 
 *
 * Purpose: Header file for MX motor driver for controlling 
 *          a Delta Tau PMAC motor through Tom Coleman's EPICS
 *          dual-ported RAM interface.
 */

#ifndef __D_PMACDP_H__
#define __D_PMACDP_H__

#include "mx_motor.h"

/* ===== MX PMAC EPICS dual-ported RAM motor record data structure ===== */

typedef struct {
	char actual_position_record_name[32];
	char requested_position_record_name[32];
	char in_position_record_name[32];
	char abort_record_name[32];
} MX_PMAC_DPR_MOTOR;

/* Define all of the interface functions. */

MX_API mx_status_type mxd_pmac_dpr_motor_initialize_type( long type );
MX_API mx_status_type mxd_pmac_dpr_motor_create_record_structures(
					MX_RECORD *record );
MX_API mx_status_type mxd_pmac_dpr_motor_finish_record_initialization(
					MX_RECORD *record );
MX_API mx_status_type mxd_pmac_dpr_motor_delete_record( MX_RECORD *record );
MX_API mx_status_type mxd_pmac_dpr_motor_print_structure(
					FILE *file, MX_RECORD *record );
MX_API mx_status_type mxd_pmac_dpr_motor_read_parms_from_hardware(
						MX_RECORD *record );
MX_API mx_status_type mxd_pmac_dpr_motor_write_parms_to_hardware(
						MX_RECORD *record );
MX_API mx_status_type mxd_pmac_dpr_motor_open( MX_RECORD *record );
MX_API mx_status_type mxd_pmac_dpr_motor_close( MX_RECORD *record );
MX_API mx_status_type mxd_pmac_dpr_motor_motor_is_busy( MX_MOTOR *motor );
MX_API mx_status_type mxd_pmac_dpr_motor_move_absolute( MX_MOTOR *motor );
MX_API mx_status_type mxd_pmac_dpr_motor_get_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_pmac_dpr_motor_set_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_pmac_dpr_motor_soft_abort( MX_MOTOR *motor );
MX_API mx_status_type mxd_pmac_dpr_motor_immediate_abort( MX_MOTOR *motor );
MX_API mx_status_type mxd_pmac_dpr_motor_positive_limit_hit( MX_MOTOR *motor );
MX_API mx_status_type mxd_pmac_dpr_motor_negative_limit_hit( MX_MOTOR *motor );
MX_API mx_status_type mxd_pmac_dpr_motor_find_home_position(MX_MOTOR *motor);

extern MX_RECORD_FUNCTION_LIST mxd_pmac_dpr_motor_record_function_list;
extern MX_MOTOR_FUNCTION_LIST mxd_pmac_dpr_motor_motor_function_list;

extern long mxd_pmac_dpr_motor_num_record_fields;
extern MX_RECORD_FIELD_DEFAULTS *mxd_pmac_dpr_motor_rfield_def_ptr;

#define MXD_PMAC_DPR_MOTOR_STANDARD_FIELDS \
  {-1, -1, "actual_position_record_name", MXFT_STRING, NULL, 1, {33}, \
	MXF_REC_TYPE_STRUCT, \
		offsetof(MX_PMAC_DPR_MOTOR, actual_position_record_name), \
	{sizeof(char)}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "requested_position_record_name", MXFT_STRING, NULL, 1, {33}, \
	MXF_REC_TYPE_STRUCT, \
		offsetof(MX_PMAC_DPR_MOTOR, requested_position_record_name), \
	{sizeof(char)}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "in_position_record_name", MXFT_STRING, NULL, 1, {33}, \
	MXF_REC_TYPE_STRUCT, \
		offsetof(MX_PMAC_DPR_MOTOR, in_position_record_name), \
	{sizeof(char)}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "abort_record_name", MXFT_STRING, NULL, 1, {33}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_PMAC_DPR_MOTOR, abort_record_name), \
	{sizeof(char)}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}

#endif /* __D_PMACDP_H__ */
