/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:   i_cmpmtr.h
 *
 * Purpose: Header for MX driver for Compumotor 6000 series motor controllers.
 */

#ifndef __I_CMPMTR_H__
#define __I_CMPMTR_H__

#include "mx_rec.h"
#include "mx_gener.h"

#include "mx_motor.h"
#include "mx_rs232.h"

/* Define the data structures used by a Compumotor interface. */

#define MX_MAX_COMPUMOTOR_AXES	4

typedef struct {
	MX_RECORD *record;

	int interface_subtype;
	MX_RECORD *rs232_record;
	long num_controllers;
	long *num_axes;

	MX_RECORD *(*motor_array)[MX_MAX_COMPUMOTOR_AXES];
} MX_COMPUMOTOR_INTERFACE;

#define MXI_COMPUMOTOR_INTERFACE_STANDARD_FIELDS \
  {-1, -1, "rs232_record", MXFT_RECORD, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_COMPUMOTOR_INTERFACE, rs232_record), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "num_controllers", MXFT_LONG, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, \
		offsetof(MX_COMPUMOTOR_INTERFACE, num_controllers), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "num_axes", MXFT_LONG, NULL, 1, {MXU_VARARGS_LENGTH}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_COMPUMOTOR_INTERFACE, num_axes), \
	{sizeof(int)}, NULL, \
		(MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY | MXFF_VARARGS)}

#ifndef MX_CR
#define MX_CR	'\015'
#define MX_LF	'\012'
#endif

MX_API mx_status_type mxi_compumotor_initialize_type( long type );
MX_API mx_status_type mxi_compumotor_create_record_structures(
							MX_RECORD *record );
MX_API mx_status_type mxi_compumotor_finish_record_initialization(
							MX_RECORD *record );
MX_API mx_status_type mxi_compumotor_delete_record( MX_RECORD *record );
MX_API mx_status_type mxi_compumotor_read_parms_from_hardware(
					MX_RECORD *record );
MX_API mx_status_type mxi_compumotor_write_parms_to_hardware(
					MX_RECORD *record );
MX_API mx_status_type mxi_compumotor_open( MX_RECORD *record );
MX_API mx_status_type mxi_compumotor_close( MX_RECORD *record );
MX_API mx_status_type mxi_compumotor_resynchronize( MX_RECORD *record );

MX_API mx_status_type mxi_compumotor_getchar( MX_GENERIC *generic,
					char *c, int flags );
MX_API mx_status_type mxi_compumotor_putchar( MX_GENERIC *generic,
					char c, int flags );
MX_API mx_status_type mxi_compumotor_read( MX_GENERIC *generic,
					void *buffer, size_t count );
MX_API mx_status_type mxi_compumotor_write( MX_GENERIC *generic,
					void *buffer, size_t count );
MX_API mx_status_type mxi_compumotor_input_is_available( MX_GENERIC *generic,
						int *input_available );
MX_API mx_status_type mxi_compumotor_discard_unread_input(
				MX_GENERIC *generic, int debug_flag );
MX_API mx_status_type mxi_compumotor_discard_unwritten_output(
				MX_GENERIC *generic, int debug_flag );

extern MX_RECORD_FUNCTION_LIST mxi_compumotor_record_function_list;
extern MX_GENERIC_FUNCTION_LIST mxi_compumotor_generic_function_list;

extern long mxi_compumotor_num_record_fields;
extern MX_RECORD_FIELD_DEFAULTS *mxi_compumotor_rfield_def_ptr;

/* === Driver specific functions === */

MX_API mx_status_type mxi_compumotor_command(
	MX_COMPUMOTOR_INTERFACE *compumotor_interface,
	char *command, char *response, int response_buffer_length,
	int debug_flag );
MX_API mx_status_type mxi_compumotor_getline(
			MX_COMPUMOTOR_INTERFACE *compumotor_interface,
			char *buffer, long buffer_size, int debug_flag );
MX_API mx_status_type mxi_compumotor_putline(
			MX_COMPUMOTOR_INTERFACE *compumotor_interface,
			char *buffer, int debug_flag );
MX_API mx_status_type mxi_compumotor_getc(
			MX_COMPUMOTOR_INTERFACE *compumotor_interface,
			char *c, int debug_flag );
MX_API mx_status_type mxi_compumotor_putc(
			MX_COMPUMOTOR_INTERFACE *compumotor_interface,
			char c, int debug_flag );

#endif /* __I_CMPMTR_H__ */
