/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 *  Name:     mtst_dos.c
 *
 *  Purpose:  Stack overflow and heap memory consistency checks.
 *            This version is for MSDOS.
 */

#include <stdio.h>

#include "mxconfig.h"

#ifdef OS_MSDOS

#include <stdlib.h>
#include <malloc.h>

#include "mx_util.h"
#include "mx_mtest.h"

#if USE_HEAP_TEST

/* The following heap tester has only been tested with Microsoft C 6.0
 * and Watcom C/386 9.5.
 *
 * This routine is relatively unique in that it returns the error codes
 * MXE_SUCCESS or MXE_CORRUPT_DATA_STRUCTURE directly rather than as
 * the 'code' member of an 'mx_status_type' structure.  This, plus the
 * use of static local variables, is to minimize the use of the
 * stack by this routine.
 *
 * It goes without saying that this routine has no chance of being
 * thread safe, which is more or less irrelevant under MSDOS anyway.
 */

MX_EXPORT long
mx_heap_test( void )
{
	static struct _heapinfo hi;
	static int heapstatus;

	hi._pentry = NULL;

	/* Walk through the heap. */

	while ( (heapstatus = _heapwalk(&hi)) == _HEAPOK ) {

		/* Do nothing. */
	}

	if ( heapstatus != _HEAPOK
	  && heapstatus != _HEAPEMPTY
	  && heapstatus != _HEAPEND
	) {
		return MXE_CORRUPT_DATA_STRUCTURE;
	} else {
		return MXE_SUCCESS;
	}
}

#endif /* USE_HEAP_TEST */

#if USE_STACK_TEST

/* The following routines are derived from the mark_stack() function
 * listed in "Encyclopedia C" by Robert A. Radcliffe, page 645.
 *
 * They report back at the time the program exits the amount of
 * stack space that was not used.
 *
 * This routine is only known to work with Microsoft C 6.0.
 */

extern char end;			/* stack bottom in crt0.asm */

static char *stack_bottom = (char *) &end;
static char stack_mark = 0xFF;		/* clean stack footprint */
static char *stack_ptr;

MX_EXPORT void
mx_stack_test_handler( void )
{
	static unsigned long unused;

	unused = 0;

	stack_ptr = stack_bottom;

	/* Look for the stack footprints. */

	while ( *stack_ptr++ == stack_mark ) {
		unused++;
	}

	fprintf(stderr,"\n\nmx_stack_test_handler():\n");
	fprintf(stderr,"  Unused stack = %lu Bytes\n\n", unused);

	return;
}

MX_EXPORT void
mx_stack_test_initialize( void )
{
	static char *stack_top;

	/* stackavail() is correct to within 256 bytes. */

	stack_top = stack_bottom + stackavail();

	stack_ptr = stack_bottom;

	/* Write the value of stack_mark to all (just about) unused
	 * stack memory.
	 */

	do {
		*stack_ptr++ = stack_mark;

	} while ( stack_ptr < stack_top );

	/* Install the exit handler. */

	atexit( mx_stack_test_handler );

	return;
}

#endif /* USE_STACK_TEST */

#endif /* OS_MSDOS */
