/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:     mx_dfile.h
 *
 * Purpose:  Header file to describe output data file support.
 */

#ifndef __MX_DFILE_H__
#define __MX_DFILE_H__

#include "mxconfig.h"
#include "mx_util.h"
#include "mx_scalr.h"

/* List of datafile types. */

#define MXDF_NONE	1
#define MXDF_CHILD	2
#define MXDF_TEXT	3
#define MXDF_SFF	4
#define MXDF_XAFS 	11

/* List of datafile_open_flag values. */

#define MXDF_OPEN_OVERWRITE	1
#define MXDF_OPEN_APPEND	2

typedef struct {
	/* Reference to the MX_SCAN that creates this datafile. */
	void *scan;

	long type;
	char *typename;
	char filename[MXU_FILENAME_LENGTH + 1];
	void *datafile_type_struct;
	void *datafile_function_list;
} MX_DATAFILE;

typedef struct {
	mx_status_type ( *open ) ( MX_DATAFILE *datafile );
	mx_status_type ( *close ) ( MX_DATAFILE *datafile );
	mx_status_type ( *write_main_header ) ( MX_DATAFILE *datafile );
	mx_status_type ( *write_segment_header ) ( MX_DATAFILE *datafile );
	mx_status_type ( *write_trailer ) ( MX_DATAFILE *datafile );
	mx_status_type ( *add_measurement_to_datafile )(MX_DATAFILE *datafile);
} MX_DATAFILE_FUNCTION_LIST;

#define MXU_DATAFILE_TYPE_NAME_LENGTH	20

typedef struct {
	long type;
	char name[MXU_DATAFILE_TYPE_NAME_LENGTH + 1];
	MX_DATAFILE_FUNCTION_LIST *datafile_function_list;
} MX_DATAFILE_TYPE_ENTRY;

MX_API mx_status_type mx_get_datafile_type_by_name(
	MX_DATAFILE_TYPE_ENTRY *datafile_type_list,
	char *name,
	MX_DATAFILE_TYPE_ENTRY **datafile_type_entry );

MX_API mx_status_type mx_get_datafile_type_by_value(
	MX_DATAFILE_TYPE_ENTRY *datafile_type_list,
	long datafile_type,
	MX_DATAFILE_TYPE_ENTRY **datafile_type_entry );

MX_API mx_status_type mx_datafile_open( MX_DATAFILE *datafile );
MX_API mx_status_type mx_datafile_close( MX_DATAFILE *datafile );
MX_API mx_status_type mx_datafile_write_main_header( MX_DATAFILE *datafile );
MX_API mx_status_type mx_datafile_write_segment_header(MX_DATAFILE *datafile);
MX_API mx_status_type mx_datafile_write_trailer( MX_DATAFILE *datafile );
MX_API mx_status_type mx_add_measurement_to_datafile( MX_DATAFILE *datafile );

MX_API mx_status_type mx_update_datafile_name( MX_DATAFILE *datafile );

/* One global variable. */

extern MX_DATAFILE_TYPE_ENTRY mx_datafile_type_list[];

#endif /* __MX_DFILE_H__ */

