/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    mx_info.c
 *
 * Purpose: Functions for displaying informational output.
 */

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <sys/types.h>
#include <time.h>

#include "mxconfig.h"
#include "mx_util.h"
#include "mx_key.h"

static void (*mx_info_output_function)( char * )
					= mx_info_default_output_function;

static void (*mx_info_dialog_function)( char *, char *, char * )
					= mx_info_default_dialog_function;

MX_EXPORT void
mx_info( char *format, ... )
{
	va_list args;
	char buffer[250];

	va_start(args, format);
	vsprintf(buffer, format, args);
	va_end(args);

	/* Display the message. */

	if ( mx_info_output_function != NULL ) {
		(*mx_info_output_function)( buffer );
	}
	return;
}

MX_EXPORT void
mx_set_info_output_function( void (*output_function)(char *) )
{
	if ( output_function == NULL ) {
		mx_info_output_function = mx_info_default_output_function;
	} else {
		mx_info_output_function = output_function;
	}
	return;
}

MX_EXPORT void
mx_info_default_output_function( char *string )
{
	fprintf( stderr, "%s\n", string );

	return;
}

/*--------------------------------------------------------------*/

MX_EXPORT void
mx_info_dialog( char *text_prompt, char *gui_prompt, char *button_label )
{
	if ( mx_info_dialog_function != NULL ) {
		( *mx_info_dialog_function )( text_prompt,
						gui_prompt, button_label );
	}
	return;
}

MX_EXPORT void
mx_set_info_dialog_function( void (*dialog_function)(char *, char *, char *) )
{
	if ( dialog_function == NULL ) {
		mx_info_dialog_function = mx_info_default_dialog_function;
	} else {
		mx_info_dialog_function = dialog_function;
	}
	return;
}

MX_EXPORT void
mx_info_default_dialog_function( char *text_prompt, char *gui_prompt,
					char *button_label )
{
	fprintf( stderr, "%s", text_prompt );

	mx_getch();

	fprintf( stderr, "\n" );

	return;
}

/*--------------------------------------------------------------*/

MX_EXPORT void
mx_startup_banner( char *string )
{
	time_t time_struct;
	struct tm *current_time;
	char buffer[80];
	char *ptr;
	size_t string_length, buffer_left;

	strcpy( buffer, "*** " );

	string_length = strlen(buffer);
	buffer_left = sizeof(buffer) - string_length;

	strncat( buffer, string, buffer_left );

	string_length = strlen(buffer);
	buffer_left = sizeof(buffer) - string_length;
	ptr = buffer + string_length;

	time( &time_struct );

	current_time = localtime( &time_struct );

#if 0
	/* EGCS gives a warning about the %c in the following.
	 * It says that in some locales the year produced will only
	 * have 2 digits.  I presume this is some Y2K test.
	 */

	strftime( ptr, buffer_left, " started at %c ***", current_time );
#else
	strftime( ptr, buffer_left,
		" started at %a %b %d %H:%M:%S %Y ***", current_time );
#endif

	mx_info( buffer );

	return;
}

