/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    mx_mca.h
 *
 * Purpose: MX system header file for multichannel analyzers.
 */

#ifndef __MX_MCA_H__
#define __MX_MCA_H__

#include "mx_rec.h"

typedef struct {
	MX_RECORD *record; /* Pointer to the MX_RECORD structure that points
			    * to this MCA.
			    */
	int busy;

	unsigned long max_num_channels;
	unsigned long current_num_channels;

	unsigned long *channel_array;

	unsigned long roi_start;
	unsigned long roi_end;
	unsigned long roi_integral;

	double real_time;
	double live_time;

	double preset_time;
} MX_MCA;

#define MX_MCA_STANDARD_FIELDS \
  {-1, -1, "max_num_channels", MXFT_ULONG, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCA, max_num_channels), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "current_num_channels", MXFT_ULONG, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCA, current_num_channels), \
	{0}, NULL, MXFF_IN_SUMMARY}, \
  \
  {-1, -1, "channel_array", MXFT_ULONG, NULL, 1, {MXU_VARARGS_LENGTH}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCA, channel_array), \
	{sizeof(unsigned long)}, NULL, MXFF_VARARGS}, \
  \
  {-1, -1, "roi_start", MXFT_ULONG, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCA, roi_start), \
	{0}, NULL, MXFF_IN_SUMMARY}, \
  \
  {-1, -1, "roi_end", MXFT_ULONG, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCA, roi_start), \
	{0}, NULL, MXFF_IN_SUMMARY}, \
  \
  {-1, -1, "roi_integral", MXFT_ULONG, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCA, roi_integral), \
	{0}, NULL, MXFF_IN_SUMMARY}

typedef struct {
	mx_status_type ( *start ) ( MX_MCA *mca );
	mx_status_type ( *stop ) ( MX_MCA *mca );
	mx_status_type ( *read ) ( MX_MCA *mca );
	mx_status_type ( *clear ) ( MX_MCA *mca );
	mx_status_type ( *busy ) ( MX_MCA *mca );
	mx_status_type ( *start_for_preset_time ) ( MX_MCA *mca );
	mx_status_type ( *get_roi ) ( MX_MCA *mca );
	mx_status_type ( *set_roi ) ( MX_MCA *mca );
	mx_status_type ( *get_roi_integral ) ( MX_MCA *mca );
	mx_status_type ( *get_num_channels ) ( MX_MCA *mca );
	mx_status_type ( *set_num_channels ) ( MX_MCA *mca );
	mx_status_type ( *get_real_time ) ( MX_MCA *mca );
	mx_status_type ( *get_live_time ) ( MX_MCA *mca );
} MX_MCA_FUNCTION_LIST;

MX_API mx_status_type mx_mca_get_pointers( MX_RECORD *mca_record,
	MX_MCA **mca, MX_MCA_FUNCTION_LIST **function_list_ptr,
	const char *calling_fname );

MX_API mx_status_type mx_mca_start( MX_RECORD *mca_record );
MX_API mx_status_type mx_mca_stop( MX_RECORD *mca_record );
MX_API mx_status_type mx_mca_read( MX_RECORD *mca_record,
					unsigned long **channel_array );
MX_API mx_status_type mx_mca_clear( MX_RECORD *mca_record );
MX_API mx_status_type mx_mca_is_busy( MX_RECORD *mca_record, int *busy );
MX_API mx_status_type mx_mca_start_for_preset_time( MX_RECORD *mca_record,
					double preset_seconds );
MX_API mx_status_type mx_mca_get_roi( MX_RECORD *mca_record,
					unsigned long *roi_start,
					unsigned long *roi_end );
MX_API mx_status_type mx_mca_set_roi( MX_RECORD *mca_record,
					unsigned long roi_start,
					unsigned long roi_end );
MX_API mx_status_type mx_mca_get_roi_integral( MX_RECORD *mca_record,
					unsigned long *roi_integral );
MX_API mx_status_type mx_mca_get_num_channels( MX_RECORD *mca_record,
					unsigned long *num_channels );
MX_API mx_status_type mx_mca_set_num_channels( MX_RECORD *mca_record,
					unsigned long num_channels );
MX_API mx_status_type mx_mca_get_real_time( MX_RECORD *mca_record,
					double *real_time );
MX_API mx_status_type mx_mca_get_live_time( MX_RECORD *mca_record,
					double *live_time );

#endif /* __MX_MCA_H__ */
