/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:     mx_net.h
 *
 * Purpose:  Header file for MX network protocol.
 *
 */

#ifndef __MX_NET_H__
#define __MX_NET_H__

typedef struct {
	MX_RECORD *record;
} MX_NETWORK_SERVER;

typedef struct {
	mx_status_type ( *receive_message ) ( MX_NETWORK_SERVER *server,
						long buffer_length,
						void *buffer );
	mx_status_type ( *send_message ) ( MX_NETWORK_SERVER *server,
						void *buffer );
} MX_NETWORK_SERVER_FUNCTION_LIST;

/* Current size of an MX network header. */

#define MX_NETWORK_HEADER_LENGTH_VALUE	( 5 * sizeof(mx_uint32_type) )

/* Definition of the fields in a header. */

#define MX_NETWORK_MAGIC		0
#define MX_NETWORK_HEADER_LENGTH	1
#define MX_NETWORK_MESSAGE_LENGTH	2
#define MX_NETWORK_MESSAGE_TYPE		3
#define MX_NETWORK_STATUS_CODE		4

/* Definition of network message types. */

#define MX_NETMSG_UNEXPECTED_ERROR		1001

#define MX_NETMSG_GET_ARRAY_ASCII_COMMAND	5001  /* client -> server */
#define MX_NETMSG_GET_ARRAY_ASCII_RESPONSE	5002  /* server -> client */
#define MX_NETMSG_PUT_ARRAY_ASCII_COMMAND	5003  /* client -> server */
#define MX_NETMSG_PUT_ARRAY_ASCII_RESPONSE	5004  /* server -> client */

#define MX_NETMSG_GET_NUM_RECORDS_COMMAND	6001  /* server -> client */
#define MX_NETMSG_GET_NUM_RECORDS_RESPONSE	6002  /* client -> server */
#define MX_NETMSG_GET_NUM_FIELDS_COMMAND	6003  /* server -> client */
#define MX_NETMSG_GET_NUM_FIELDS_RESPONSE	6004  /* client -> server */
#define MX_NETMSG_GET_RECORD_NAMES_COMMAND	6005  /* server -> client */
#define MX_NETMSG_GET_RECORD_NAMES_RESPONSE	6006  /* client -> server */
#define MX_NETMSG_GET_FIELD_NAMES_COMMAND	6007  /* server -> client */
#define MX_NETMSG_GET_FIELD_NAMES_RESPONSE	6008  /* client -> server */
#define MX_NETMSG_GET_FIELD_TYPE_COMMAND	6009  /* client -> server */
#define MX_NETMSG_GET_FIELD_TYPE_RESPONSE	6010  /* client -> server */

/* The MX_NETWORK_MAGIC header field should always contain the
 * MX_NETWORK_MAGIC_VALUE.  If it does not, there has been
 * a protocol error somewhere.
 */

#define MX_NETWORK_MAGIC_VALUE		0xe7e9fcfe

MX_API mx_status_type mx_network_receive_message( MX_RECORD *server_record,
				long buffer_length, void *buffer );

MX_API mx_status_type mx_network_send_message( MX_RECORD *server_record,
					void *buffer );

MX_API void mx_network_display_message_buffer( void *buffer );

#define mx_get( s, r, d, v ) \
		mx_get_array_ascii( (s), (r), (d), 0, NULL, (v) )

#define mx_put( s, r, d, v ) \
		mx_put_array_ascii( (s), (r), (d), 0, NULL, (v) )

MX_API mx_status_type mx_get_array_ascii( MX_RECORD *server_record,
			char *remote_record_field_name,
			long datatype, 
			long num_dimensions,
			long *dimension,
			void *value );

MX_API mx_status_type mx_put_array_ascii( MX_RECORD *server_record,
			char *remote_record_field_name,
			long datatype, 
			long num_dimensions,
			long *dimension,
			void *value );

MX_API mx_status_type mx_get_field_array_ascii( MX_RECORD *server_record,
			char *remote_record_field_name,
			MX_RECORD_FIELD *local_field,
			void *value_ptr );

MX_API mx_status_type mx_put_field_array_ascii( MX_RECORD *server_record,
			char *remote_record_field_name,
			MX_RECORD_FIELD *local_field,
			void *value_ptr );

MX_API mx_status_type mx_get_num_records( MX_RECORD *server_record,
			long superclass,
			long class,
			long type,
			long *num_records );

MX_API mx_status_type mx_get_num_fields( MX_RECORD *server_record,
			long *num_fields );

MX_API mx_status_type mx_get_record_names( MX_RECORD *server_record,
			long superclass,
			long class,
			long type,
			long max_records,
			long *num_records,
			char **record_name_array );

MX_API mx_status_type mx_get_field_names( MX_RECORD *server_record,
			char *remote_record_name,
			long max_fields,
			long *num_fields,
			char **field_name_array );

MX_API mx_status_type mx_get_field_type( MX_RECORD *server_record,
			char *remote_record_field_name,
			long max_dimensions,
			long *datatype,
			long *num_dimensions,
			long *dimension_array );

#endif /* __MX_NET_H__ */
