/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:     mx_plot.h
 *
 * Purpose:  Header file to define functions to plot data during a scan.
 */

#ifndef __MX_PLOT_H__
#define __MX_PLOT_H__

#include "mxconfig.h"
#include "mx_util.h"
#include "mx_scalr.h"

/* List of plot types. */

#define MXP_NONE	1
#define MXP_CHILD	2
#define MXP_GNUPLOT	3
#define MXP_GNUXAFS	4

#define MXU_PLOT_TYPE_NAME_LENGTH	20

typedef struct {
	/* Reference to the MX_SCAN that invokes this plot. */
	void *scan;

	long type;
	char *typename;
	char *plot_arguments;
	void *plot_type_struct;
	void *plot_function_list;
	unsigned long section_number;
} MX_PLOT;

typedef struct {
	mx_status_type ( *open ) ( MX_PLOT *plot );
	mx_status_type ( *close ) ( MX_PLOT *plot );
	mx_status_type ( *add_measurement_to_plot ) ( MX_PLOT *plot );
	mx_status_type ( *set_x_range ) ( MX_PLOT *plot,
					double x_min, double x_max );
	mx_status_type ( *set_y_range ) ( MX_PLOT *plot,
					double y_min, double y_max );
	mx_status_type ( *start_plot_section ) ( MX_PLOT *plot );
} MX_PLOT_FUNCTION_LIST;

typedef struct {
	long type;
	char name[MXU_PLOT_TYPE_NAME_LENGTH + 1];
	MX_PLOT_FUNCTION_LIST *plot_function_list;
} MX_PLOT_TYPE_ENTRY;

MX_API mx_status_type mx_get_plot_type_by_name(
	MX_PLOT_TYPE_ENTRY *plot_type_list,
	char *name,
	MX_PLOT_TYPE_ENTRY **plot_type_entry );

MX_API mx_status_type mx_get_plot_type_by_value(
	MX_PLOT_TYPE_ENTRY *plot_type_list,
	long plot_type,
	MX_PLOT_TYPE_ENTRY **plot_type_entry );

MX_API mx_status_type mx_plot_open( MX_PLOT *plot );
MX_API mx_status_type mx_plot_close( MX_PLOT *plot );
MX_API mx_status_type mx_add_measurement_to_plot( MX_PLOT *plot );
MX_API mx_status_type mx_plot_set_x_range( MX_PLOT *plot,
				double x_min, double x_max );
MX_API mx_status_type mx_plot_set_y_range( MX_PLOT *plot,
				double y_min, double y_max );
MX_API mx_status_type mx_plot_start_plot_section( MX_PLOT *plot );

MX_API int mx_plotting_is_enabled( MX_RECORD *record );
MX_API void mx_set_plot_enable( MX_RECORD *record, int enable );

/* One global variable. */

extern MX_PLOT_TYPE_ENTRY mx_plot_type_list[];

#endif /* __MX_PLOT_H__ */

