/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name: mx_scalr.h
 *
 * Purpose: MX system header file for scalers.
 */

#ifndef __MX_SCALR_H__
#define __MX_SCALR_H__

#include "mx_rec.h"

typedef struct {
	MX_RECORD *record; /* Pointer to the MX_RECORD structure that points
	                    * to this scaler.
	                    */
	long value;
	double dark_current;        /* Expressed in counts per second. */
	int  subtract_dark_current;
	int  automatic_overflow_check;
	int  overflow_set;
} MX_SCALER;

#define MXLV_SCL_VALUE			6001
#define MXLV_SCL_OVERFLOW_SET		6002

#define MX_SCALER_STANDARD_FIELDS \
  {MXLV_SCL_VALUE, -1, "value", MXFT_LONG, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_SCALER, value), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "dark_current", MXFT_DOUBLE, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_SCALER, dark_current), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "subtract_dark_current", MXFT_INT, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_SCALER, subtract_dark_current), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {MXLV_SCL_OVERFLOW_SET, -1, "overflow_set", MXFT_INT, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_SCALER, overflow_set), \
	{0}, NULL, 0 }

/*
 * The structure type MX_SCALER_FUNCTION_LIST contains a list of 
 * pointers to functions that vary from scaler type to scaler type.
 */

typedef struct {
	mx_status_type ( *clear ) ( MX_SCALER *scaler );
	mx_status_type ( *overflow_set ) ( MX_SCALER *scaler );
	mx_status_type ( *read ) ( MX_SCALER *scaler );
	mx_status_type ( *read_and_clear ) ( MX_SCALER *scaler );
	mx_status_type ( *write ) ( MX_SCALER *scaler );
} MX_SCALER_FUNCTION_LIST;

MX_API mx_status_type mx_scaler_get_pointers( MX_RECORD *scaler_record,
	MX_SCALER **scaler, MX_SCALER_FUNCTION_LIST **function_list_ptr,
	const char *calling_fname );

MX_API mx_status_type mx_scaler_finish_record_initialization(
							MX_RECORD *record );

MX_API mx_status_type mx_scaler_clear( MX_RECORD *scaler_record );
MX_API mx_status_type mx_scaler_overflow_set( MX_RECORD *scaler_record,
							int *overflow_set);
MX_API mx_status_type mx_scaler_read( MX_RECORD *scaler_record, long *value );
MX_API mx_status_type mx_scaler_read_and_clear( MX_RECORD *scaler_record,
							long *value );
MX_API mx_status_type mx_scaler_write( MX_RECORD *scaler_record, long value );
MX_API mx_status_type mx_scaler_get_dark_current( MX_RECORD *scaler_record,
							double *dark_current );
MX_API mx_status_type mx_scaler_set_dark_current( MX_RECORD *scaler_record,
							double dark_current );

#endif /* __MX_SCALR_H__ */
