/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    mx_scseg.h
 *
 * Purpose: Header file for MX_SEGMENTED_SCAN scan records.
 */

#ifndef _MX_SCSEG_H_
#define _MX_SCSEG_H_

typedef struct {
	MX_SCAN *scan;	/* Pointer to the parent scan superclass structure. */

	long num_segments;
	MX_RECORD **scan_segment_record_array;

	void *record_type_struct;
} MX_SEGMENTED_SCAN;

typedef struct {
	mx_status_type ( *create_record_structures )
			( MX_RECORD *, MX_SCAN *, MX_SEGMENTED_SCAN * );
	mx_status_type ( *finish_record_initialization ) ( MX_RECORD * );
	mx_status_type ( *delete_record ) ( MX_RECORD * );
} MX_SEGMENTED_SCAN_FUNCTION_LIST;

MX_API mx_status_type mxs_segmented_scan_initialize_type( long type );
MX_API mx_status_type mxs_segmented_scan_create_record_structures(
							MX_RECORD *record );
MX_API mx_status_type mxs_segmented_scan_finish_record_initialization(
							MX_RECORD *record );
MX_API mx_status_type mxs_segmented_scan_delete_record( MX_RECORD *record );
MX_API mx_status_type mxs_segmented_scan_dummy_function( MX_RECORD *record );

MX_API mx_status_type mxs_segmented_scan_prepare_for_scan_start(MX_SCAN *scan);
MX_API mx_status_type mxs_segmented_scan_perform_scan( MX_SCAN *scan );

extern MX_RECORD_FUNCTION_LIST mxs_segmented_scan_record_function_list;
extern MX_SCAN_FUNCTION_LIST mxs_segmented_scan_scan_function_list;

#define MX_SEGMENTED_SCAN_STANDARD_FIELDS \
  {-1, -1, "num_segments", MXFT_LONG, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_SEGMENTED_SCAN, num_segments), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  {-1, -1, "scan_segment_array", MXFT_RECORD, NULL, 1, {MXU_VARARGS_LENGTH}, \
	MXF_REC_CLASS_STRUCT, \
	offsetof(MX_SEGMENTED_SCAN, scan_segment_record_array), \
	{sizeof(MX_RECORD *)}, NULL, (MXFF_IN_DESCRIPTION | MXFF_VARARGS)}

#endif /* _MX_SCSEG_H_ */

