/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    mx_timer.h
 *
 * Purpose: MX system header file for timers.
 */

#ifndef __MX_TIMER_H__
#define __MX_TIMER_H__

#include "mx_rec.h"

typedef struct {
	MX_RECORD *record; /* Pointer to the MX_RECORD structure that points
                            * to this timer.
                            */
	int busy;
	double start;
	int stop;
} MX_TIMER;

#define MXLV_TIM_BUSY	7001
#define MXLV_TIM_START	7002
#define MXLV_TIM_STOP	7003

#define MX_TIMER_STANDARD_FIELDS \
  {MXLV_TIM_BUSY, -1, "busy", MXFT_INT, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_TIMER, busy), \
	{0}, NULL, 0 }, \
  \
  {MXLV_TIM_START, -1, "start", MXFT_DOUBLE, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_TIMER, start), \
	{0}, NULL, 0 }, \
  \
  {MXLV_TIM_STOP, -1, "stop", MXFT_INT, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_TIMER, stop), \
	{0}, NULL, 0 }

typedef struct {
	mx_status_type ( *timer_is_busy )( MX_TIMER *timer, int *busy );
	mx_status_type ( *start )( MX_TIMER *timer, double seconds );
	mx_status_type ( *stop )( MX_TIMER *timer, double *seconds_left );
	mx_status_type ( *get_current_time )( MX_TIMER *timer, double *seconds);
} MX_TIMER_FUNCTION_LIST;

MX_API mx_status_type mx_timer_finish_record_initialization(MX_RECORD *record);

MX_API mx_status_type mx_timer_is_busy( MX_RECORD *timer_record, int *busy );
MX_API mx_status_type mx_timer_start( MX_RECORD *timer_record, double seconds);
MX_API mx_status_type mx_timer_stop( MX_RECORD *timer_record,
						double *seconds_left );
MX_API mx_status_type mx_timer_get_current_time( MX_RECORD *timer_record,
						double *seconds );

extern MX_RECORD_FUNCTION_LIST mx_timer_record_function_list;

#endif /* __MX_TIMER_H__ */

