/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    mx_vepic.c
 *
 * Purpose: Support for EPICS variables in MX database description files.
 */

#include <stdio.h>
#include <stdlib.h>

#include "mxconfig.h"

#if HAVE_EPICS

/* The following three include files are from EPICS. */

#include "tsDefs.h"
#include "cadef.h"
#include "ezca.h"

#include "mx_rec.h"
#include "mx_epics.h"
#include "mx_var.h"
#include "mx_vepic.h"

MX_RECORD_FUNCTION_LIST mxv_epics_variable_record_function_list = {
	mxv_epics_variable_initialize_type,
	NULL,
	NULL,
	mxv_epics_variable_create_record_structures,
	mxv_epics_variable_finish_record_initialization,
	mxv_epics_variable_delete_record,
	NULL,
	NULL,
	mx_receive_variable,
	mxv_epics_variable_dummy_function,
	mx_receive_variable,
	mxv_epics_variable_dummy_function
};

MX_VARIABLE_FUNCTION_LIST mxv_epics_variable_variable_function_list = {
	mxv_epics_variable_send_variable,
	mxv_epics_variable_receive_variable
};

/********************************************************************/

MX_RECORD_FIELD_DEFAULTS mxv_epics_string_variable_defaults[] = {
	MX_RECORD_STANDARD_FIELDS,
	MX_EPICS_VARIABLE_STANDARD_FIELDS,
	MX_VARIABLE_STANDARD_FIELDS,
	MX_STRING_VARIABLE_STANDARD_FIELDS
};

long mxv_epics_string_variable_num_record_fields
			= sizeof( mxv_epics_string_variable_defaults )
			/ sizeof( mxv_epics_string_variable_defaults[0] );

MX_RECORD_FIELD_DEFAULTS *mxv_epics_string_variable_def_ptr
			= &mxv_epics_string_variable_defaults[0];

/* ==== */

MX_RECORD_FIELD_DEFAULTS mxv_epics_char_variable_defaults[] = {
	MX_RECORD_STANDARD_FIELDS,
	MX_EPICS_VARIABLE_STANDARD_FIELDS,
	MX_VARIABLE_STANDARD_FIELDS,
	MX_CHAR_VARIABLE_STANDARD_FIELDS
};

long mxv_epics_char_variable_num_record_fields
			= sizeof( mxv_epics_char_variable_defaults )
			/ sizeof( mxv_epics_char_variable_defaults[0] );

MX_RECORD_FIELD_DEFAULTS *mxv_epics_char_variable_def_ptr
			= &mxv_epics_char_variable_defaults[0];

/* ==== */

MX_RECORD_FIELD_DEFAULTS mxv_epics_uchar_variable_defaults[] = {
	MX_RECORD_STANDARD_FIELDS,
	MX_EPICS_VARIABLE_STANDARD_FIELDS,
	MX_VARIABLE_STANDARD_FIELDS,
	MX_UCHAR_VARIABLE_STANDARD_FIELDS
};

long mxv_epics_uchar_variable_num_record_fields
			= sizeof( mxv_epics_uchar_variable_defaults )
			/ sizeof( mxv_epics_uchar_variable_defaults[0] );

MX_RECORD_FIELD_DEFAULTS *mxv_epics_uchar_variable_def_ptr
			= &mxv_epics_uchar_variable_defaults[0];

/* ==== */

MX_RECORD_FIELD_DEFAULTS mxv_epics_short_variable_defaults[] = {
	MX_RECORD_STANDARD_FIELDS,
	MX_EPICS_VARIABLE_STANDARD_FIELDS,
	MX_VARIABLE_STANDARD_FIELDS,
	MX_SHORT_VARIABLE_STANDARD_FIELDS
};

long mxv_epics_short_variable_num_record_fields
			= sizeof( mxv_epics_short_variable_defaults )
			/ sizeof( mxv_epics_short_variable_defaults[0] );

MX_RECORD_FIELD_DEFAULTS *mxv_epics_short_variable_def_ptr
			= &mxv_epics_short_variable_defaults[0];

/* ==== */

MX_RECORD_FIELD_DEFAULTS mxv_epics_ushort_variable_defaults[] = {
	MX_RECORD_STANDARD_FIELDS,
	MX_EPICS_VARIABLE_STANDARD_FIELDS,
	MX_VARIABLE_STANDARD_FIELDS,
	MX_USHORT_VARIABLE_STANDARD_FIELDS
};

long mxv_epics_ushort_variable_num_record_fields
			= sizeof( mxv_epics_ushort_variable_defaults )
			/ sizeof( mxv_epics_ushort_variable_defaults[0] );

MX_RECORD_FIELD_DEFAULTS *mxv_epics_ushort_variable_def_ptr
			= &mxv_epics_ushort_variable_defaults[0];

/* ==== */

MX_RECORD_FIELD_DEFAULTS mxv_epics_int_variable_defaults[] = {
	MX_RECORD_STANDARD_FIELDS,
	MX_EPICS_VARIABLE_STANDARD_FIELDS,
	MX_VARIABLE_STANDARD_FIELDS,
	MX_INT_VARIABLE_STANDARD_FIELDS
};

long mxv_epics_int_variable_num_record_fields
			= sizeof( mxv_epics_int_variable_defaults )
			/ sizeof( mxv_epics_int_variable_defaults[0] );

MX_RECORD_FIELD_DEFAULTS *mxv_epics_int_variable_def_ptr
			= &mxv_epics_int_variable_defaults[0];

/* ==== */

MX_RECORD_FIELD_DEFAULTS mxv_epics_uint_variable_defaults[] = {
	MX_RECORD_STANDARD_FIELDS,
	MX_EPICS_VARIABLE_STANDARD_FIELDS,
	MX_VARIABLE_STANDARD_FIELDS,
	MX_UINT_VARIABLE_STANDARD_FIELDS
};

long mxv_epics_uint_variable_num_record_fields
			= sizeof( mxv_epics_uint_variable_defaults )
			/ sizeof( mxv_epics_uint_variable_defaults[0] );

MX_RECORD_FIELD_DEFAULTS *mxv_epics_uint_variable_def_ptr
			= &mxv_epics_uint_variable_defaults[0];

/* ==== */

MX_RECORD_FIELD_DEFAULTS mxv_epics_long_variable_defaults[] = {
	MX_RECORD_STANDARD_FIELDS,
	MX_EPICS_VARIABLE_STANDARD_FIELDS,
	MX_VARIABLE_STANDARD_FIELDS,
	MX_LONG_VARIABLE_STANDARD_FIELDS
};

long mxv_epics_long_variable_num_record_fields
			= sizeof( mxv_epics_long_variable_defaults )
			/ sizeof( mxv_epics_long_variable_defaults[0] );

MX_RECORD_FIELD_DEFAULTS *mxv_epics_long_variable_def_ptr
			= &mxv_epics_long_variable_defaults[0];

/* ==== */

MX_RECORD_FIELD_DEFAULTS mxv_epics_ulong_variable_defaults[] = {
	MX_RECORD_STANDARD_FIELDS,
	MX_EPICS_VARIABLE_STANDARD_FIELDS,
	MX_VARIABLE_STANDARD_FIELDS,
	MX_ULONG_VARIABLE_STANDARD_FIELDS
};

long mxv_epics_ulong_variable_num_record_fields
			= sizeof( mxv_epics_ulong_variable_defaults )
			/ sizeof( mxv_epics_ulong_variable_defaults[0] );

MX_RECORD_FIELD_DEFAULTS *mxv_epics_ulong_variable_def_ptr
			= &mxv_epics_ulong_variable_defaults[0];

/* ==== */

MX_RECORD_FIELD_DEFAULTS mxv_epics_float_variable_defaults[] = {
	MX_RECORD_STANDARD_FIELDS,
	MX_EPICS_VARIABLE_STANDARD_FIELDS,
	MX_VARIABLE_STANDARD_FIELDS,
	MX_FLOAT_VARIABLE_STANDARD_FIELDS
};

long mxv_epics_float_variable_num_record_fields
			= sizeof( mxv_epics_float_variable_defaults )
			/ sizeof( mxv_epics_float_variable_defaults[0] );

MX_RECORD_FIELD_DEFAULTS *mxv_epics_float_variable_def_ptr
			= &mxv_epics_float_variable_defaults[0];

/* ==== */

MX_RECORD_FIELD_DEFAULTS mxv_epics_double_variable_defaults[] = {
	MX_RECORD_STANDARD_FIELDS,
	MX_EPICS_VARIABLE_STANDARD_FIELDS,
	MX_VARIABLE_STANDARD_FIELDS,
	MX_DOUBLE_VARIABLE_STANDARD_FIELDS
};

long mxv_epics_double_variable_num_record_fields
			= sizeof( mxv_epics_double_variable_defaults )
			/ sizeof( mxv_epics_double_variable_defaults[0] );

MX_RECORD_FIELD_DEFAULTS *mxv_epics_double_variable_def_ptr
			= &mxv_epics_double_variable_defaults[0];

/********************************************************************/

MX_EXPORT mx_status_type
mxv_epics_variable_initialize_type( long record_type )
{
	return mx_variable_initialize_type( record_type );
}

MX_EXPORT mx_status_type
mxv_epics_variable_create_record_structures( MX_RECORD *record )
{
	const char fname[] = "mxv_epics_variable_create_record_structures()";

	MX_VARIABLE *variable_struct;
	MX_EPICS_VARIABLE *epics_variable;

	/* Allocate memory for the necessary structures. */

	variable_struct = (MX_VARIABLE *) malloc( sizeof(MX_VARIABLE) );

	if ( variable_struct == (MX_VARIABLE *) NULL ) {
		return mx_error( MXE_OUT_OF_MEMORY, fname,
		"Can't allocate memory for MX_VARIABLE structure." );
	}

	epics_variable = (MX_EPICS_VARIABLE *)
				malloc( sizeof(MX_EPICS_VARIABLE) );

	if ( epics_variable == (MX_EPICS_VARIABLE *) NULL ) {
		return mx_error( MXE_OUT_OF_MEMORY, fname,
		"Can't allocate memory for MX_EPICS_VARIABLE structure." );
	}

	/* Now set up the necessary pointers. */

	variable_struct->record = record;

	record->record_superclass_struct = variable_struct;
	record->record_class_struct = epics_variable;
	record->record_type_struct = NULL;

	record->superclass_specific_function_list =
				&mxv_epics_variable_variable_function_list;
	record->class_specific_function_list = NULL;

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxv_epics_variable_finish_record_initialization( MX_RECORD *record )
{
	const char fname []
		= "mxv_epics_variable_finish_record_initialization()";

	MX_EPICS_VARIABLE *epics_variable;
	MX_RECORD_FIELD *value_field;
	char mx_error_message[150];
	char *ezca_error_message;
	int ezca_status, num_elements;
	char ezca_type;
	mx_status_type status;

	/* EPICS variables may only be associated with 1-dimensional
	 * MX variables.
	 *
	 * Test this by looking at the number of dimensions in the 
	 * 'value' field.
	 */

	status = mx_find_record_field( record, "value", &value_field );

	if ( status.code != MXE_SUCCESS )
		return status;

	if ( value_field->num_dimensions != 1 ) {
		return mx_error( MXE_ILLEGAL_ARGUMENT, fname,
"EPICS variables may only be associated with 1-dimensional MX variables.  "
"The field '%s.value' has %ld variables.",
			record->name,
			value_field->num_dimensions );
	}

	/* Store the EZCA type associated with this MX field's datatype. */

	status = mx_epics_convert_mx_type_to_ezca_type(
			value_field->datatype, &ezca_type );

	if ( status.code != MXE_SUCCESS )
		return status;

	epics_variable = (MX_EPICS_VARIABLE *) record->record_class_struct;

	epics_variable->ezca_type = ezca_type;

	/* Turn off automatic printing of EZCA error messages, since we
	 * want to intercept them ourselves.
	 */

	ezcaAutoErrorMessageOff();

	/* Find out the number of elements in the EPICS process variable. */

	ezca_status = ezcaGetNelem( epics_variable->pvname, &num_elements );

	if ( ezca_status == EZCA_OK ) {

		epics_variable->num_elements = num_elements;

	} else {
		if ( ezca_status == EZCA_NOTCONNECTED ) {
			sprintf( mx_error_message,
			"The EPICS process variable '%s' used by "
			"MX variable '%s' cannot be found at present.",
				epics_variable->pvname,
				record->name );
		} else {
			ezcaGetErrorString( NULL, &ezca_error_message );

			sprintf( mx_error_message,
			"Error getting number of elements for EPICS PV='%s'.  "
			"EZCA status = %d (%s)",
				epics_variable->pvname, ezca_status,
				ezca_error_message );
		}
		(void) mx_error(MXE_DEVICE_IO_ERROR, fname, mx_error_message);

		/* If the attempt to get the number of elements from EPICS
		 * failed, use the size of the local record as a best guess.
		 */

		epics_variable->num_elements = value_field->dimension[0];
	}
	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxv_epics_variable_delete_record( MX_RECORD *record )
{
	const char fname[] = "mxv_epics_variable_delete_record()";

	return mx_error( MXE_NOT_YET_IMPLEMENTED, fname,
		"This function is not yet implemented." );
}

MX_EXPORT mx_status_type
mxv_epics_variable_dummy_function( MX_RECORD *record )
{
	/* This function serves as a placeholder for functions that are
	 * not needed by this variable class.
	 */

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxv_epics_variable_send_variable( MX_VARIABLE *variable )
{
	const char fname[] = "mxv_epics_variable_send_variable()";

	MX_EPICS_VARIABLE *epics_variable;
	MX_RECORD_FIELD *value_field;
	void *value_ptr;
	char mx_error_message[150];
	char *ezca_error_message;
	int num_elements, ezca_status;
	mx_status_type status;

	epics_variable = (MX_EPICS_VARIABLE *)
				variable->record->record_class_struct;

	status = mx_find_record_field(variable->record, "value", &value_field);

	if ( status.code != MXE_SUCCESS )
		return status;

	if ( value_field->dimension[0] >= epics_variable->num_elements ) {
		num_elements = epics_variable->num_elements;
	} else {
		num_elements = value_field->dimension[0];
	}

	value_ptr = mx_get_field_value_pointer( value_field );

	ezca_status = ezcaPut( epics_variable->pvname,
				epics_variable->ezca_type,
				num_elements,
				value_ptr );

	if ( ezca_status != EZCA_OK ) {
		ezcaGetErrorString( NULL, &ezca_error_message );

		sprintf( mx_error_message,
		"Error changing current value of EPICS process variable '%s'.  "
		"EZCA status = %d (%s)",
			epics_variable->pvname, ezca_status,
			ezca_error_message );

		return mx_error(MXE_DEVICE_IO_ERROR, fname, mx_error_message);
	}
	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxv_epics_variable_receive_variable( MX_VARIABLE *variable )
{
	const char fname[] = "mxv_epics_variable_receive_variable()";

	MX_EPICS_VARIABLE *epics_variable;
	MX_RECORD_FIELD *value_field;
	void *value_ptr;
	char mx_error_message[150];
	char *ezca_error_message;
	int num_elements, ezca_status;
	mx_status_type status;

	epics_variable = (MX_EPICS_VARIABLE *)
				variable->record->record_class_struct;

	status = mx_find_record_field(variable->record, "value", &value_field);

	if ( status.code != MXE_SUCCESS )
		return status;

	if ( value_field->dimension[0] >= epics_variable->num_elements ) {
		num_elements = epics_variable->num_elements;
	} else {
		num_elements = value_field->dimension[0];
	}

	value_ptr = mx_get_field_value_pointer( value_field );

	ezca_status = ezcaGet( epics_variable->pvname,
				epics_variable->ezca_type,
				num_elements,
				value_ptr );

	if ( ezca_status != EZCA_OK ) {
		ezcaGetErrorString( NULL, &ezca_error_message );

		sprintf( mx_error_message,
		"Error getting current value of EPICS process variable '%s'.  "
		"EZCA status = %d (%s)",
			epics_variable->pvname, ezca_status,
			ezca_error_message );

		return mx_error(MXE_DEVICE_IO_ERROR, fname, mx_error_message);
	}
	return MX_SUCCESSFUL_RESULT;
}

#endif /* HAVE_EPICS */

