/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/* movrwrit.c --- Check to see if any data files may be overwritten. */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#include "motor.h"
#include "mdialog.h"
#include "mx_var.h"

int
motor_check_for_datafile_name_collision( MX_SCAN *scan )
{
	char *datafile_name;
	char buffer[20];
	int status, saved_errno, string_length;

	/* Check to see if this test has been turned off. */

	if ( motor_overwrite_flag == TRUE )
		return SUCCESS;

	/* MXDF_NONE scans do not have a datafile. */

	if ( scan->datafile->type == MXDF_NONE )
		return SUCCESS;

	/* The following tests are subject to race conditions. */

	datafile_name = scan->datafile->filename;

	status = access( datafile_name, F_OK );

	saved_errno = errno;

	if ( status != 0 ) {
		switch( saved_errno ) {
		case ENOENT:
			/* The file does not exist, so we return success. */

			return SUCCESS;
			break;
		default:
			fprintf( output,
"scan: Unexpected error with filename '%s'.  Errno = %d, error text = '%s'\n",
				datafile_name, saved_errno,
				strerror( saved_errno ) );
			return FAILURE;
			break;
		}
	}

	/* If we got here, the file exists.  Do we have write permission? */

	status = access( datafile_name, W_OK );

	saved_errno = errno;

	if ( status != 0 ) {
		switch( saved_errno ) {
		case EACCES:
			fprintf( output,
"scan: The output datafile named '%s' already exists\n",
				datafile_name );
			fprintf( output,
		"      and is write protected.  Aborting the scan.\n" );
			return FAILURE;
			break;
		default:
			fprintf( output,
"scan: Unexpected write permission error with filename '%s'.  "
"Errno = %d, error text = '%s'\n",
			datafile_name, saved_errno,
			strerror( saved_errno ) );
			return FAILURE;
			break;
		}
	}

	/* If we got here, the file exists and is writeable. */

	fprintf( output, "\a" );

	while( 1 ) {
		fprintf( output, "The output datafile '%s' already exists.\n",
			datafile_name );

		string_length = sizeof(buffer) - 1;

		status = motor_get_string( output,
			" Overwrite it (Y/N) ? ",
			NULL, &string_length, buffer );

		if ( status != FAILURE ) {
			switch( buffer[0] ) {
			case 'Y':
			case 'y':
				/* The user has requested that we overwrite
				 * the datafile, so we return SUCCESS.
				 */
				return SUCCESS;
				break;
			case 'N':
			case 'n':
				fprintf( output,
	"\nThe datafile will not be overwritten.  Aborting the scan.\n\n" );
				return FAILURE;
				break;
			case 'Q':
			case 'q':
				fprintf( output, "\nScan aborted.\n\n" );
				return FAILURE;
				break;
			default:
				fprintf( output,
	"\nPlease enter either Y or N or Q to abort the scan.\n\n" );
				break;
			}
		}
	}
}

