/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/* mscan.c --- Scan routine for the simple motor scan program. */

#include <stdio.h>

#include "motor.h"

int
motor_scan_fn( int argc, char *argv[] )
{
	MX_RECORD *record;
	MX_SCAN *scan;
	int status;
	mx_status_type mx_status;

	if ( argc != 3 ) {
		fprintf(output, "Usage: scan 'scan_name'.\n");
		return FAILURE;
	}

	/* Get the record for this scan name. */

	record = mx_get_record( motor_record_list, argv[2] );

	if ( record == (MX_RECORD *) NULL ) {
		fprintf( output,
			"No scan record named '%s' was found.\n", argv[2] );
		return FAILURE;
	}

	/* Is this a scan record? */

	if ( record->superclass != MXR_SCAN ) {
		fprintf( output,
			"'%s' is not a scan record.  Record class = %ld\n",
			argv[2], record->superclass );
		return FAILURE;
	}

	scan = (MX_SCAN *) (record->record_superclass_struct);

	if ( scan == (MX_SCAN *) NULL ) {
		fprintf( output,
			"MX_SCAN structure for scan '%s' is NULL.\n",
			argv[2] );
		return FAILURE;
	}

	/* Is this a child scan?  Child scans should not be directly invoked.*/

	if ( scan->parent_scan_record != NULL ) {
		fprintf( output,
	"'%s' is a child scan record and cannot be invoked directly.\n",
			argv[2] );
		fprintf( output,
	"Its parent scan appears to be '%s'.  Invoke that instead.\n",
			scan->parent_scan_record->name );
		return FAILURE;
	}

	/* Does the data file specified for this scan already exist? */

	status = motor_check_for_datafile_name_collision( scan );

	if ( status != SUCCESS )
		return status;

	/* Prompt for changes to the scan header. */

	status = motor_prompt_for_scan_header( scan );

	if ( status != SUCCESS )
		return status;

	/* Invoke the scan function. */

	mx_status = mx_perform_scan( scan );

	if ( mx_status.code != MXE_SUCCESS )
		return FAILURE;

	return SUCCESS;
}

