/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    ms_epics.h
 *
 * Purpose: Header file for handling of EPICS MX server events.
 */

#ifndef __MS_EPICS_H__
#define __MS_EPICS_H__

/* Note that 'chid' is typedef'ed in the EPICS include file "cadef.h" */

struct mx_epics_channel_type {
	chid channel_id;		/* EPICS channel id. */
	evid event_id;			/* EPICS event id. */

	mx_status_type ( *process_fn )
		( struct mx_epics_channel_type *channel,
		  int direction,
		  void *value_ptr );
	MX_RECORD *mailbox_record;
	MX_RECORD *mx_record;
	MX_RECORD_FIELD *mx_record_field;

	int discard_events;
	int discard_next_event;

	int put_callback_in_progress;
	TS_STAMP last_put_time;

	int waiting_for_internal_callback;
};

typedef struct {
	char epics_field_name[33];
	char mx_field_name[MXU_FIELD_NAME_LENGTH];
	mx_status_type ( *process_fn )
		( struct mx_epics_channel_type *channel,
		  int direction,
		  void *value_ptr );
} MX_EPICS_CHANNEL_DEFAULTS;

typedef struct mx_epics_channel_type MX_EPICS_CHANNEL;

#define mxsrv_epics_convert_dbr_type_to_dbr_time_type( x ) \
					( (x) + DBR_TIME_STRING )

extern mx_status_type mxsrv_epics_init(
                                MX_RECORD *record_list,
                                MXSRV_SOCKET_HANDLER_LIST *socket_handler_list,
                                MXSRV_EVENT_HANDLER *event_handler );

extern mx_status_type mxsrv_epics_process_event(
                                MX_RECORD *record_list,
                                MXSRV_SOCKET_HANDLER *socket_handler,
                                MXSRV_SOCKET_HANDLER_LIST *socket_handler_list,
                                MXSRV_EVENT_HANDLER *event_handler );

extern mx_status_type mxsrv_epics_channel_access_initialize(
			MX_RECORD *record_list,
			MXSRV_SOCKET_HANDLER_LIST *socket_handler_list );

extern void mxsrv_epics_fd_register_fn( void *userarg, int fd, int opened );

extern mx_status_type mxsrv_epics_convert_mx_type_to_epics_type(
				long mx_type, chtype *epics_type );

extern mx_status_type mxsrv_epics_mx_value_changed_callback(
				MX_RECORD_FIELD *record_field,
				void *callback_arguments );

extern void mxsrv_epics_value_changed_callback(
				struct event_handler_args args );

extern void mxsrv_epics_change_connection_callback(
				struct connection_handler_args args );

#define MXSRV_READ_FROM_EPICS	1
#define MXSRV_WRITE_TO_EPICS	2

extern mx_status_type mxsrv_epics_standard_process_fn(
				MX_EPICS_CHANNEL *channel,
				int direction,
				void *value_ptr );

extern mx_status_type mxsrv_epics_array_put(
				chtype epics_type,
				unsigned long element_count,
				MX_EPICS_CHANNEL *channel,
				void *field_value_ptr );

extern void mxsrv_epics_internal_put_callback(
				struct event_handler_args args );

extern void mxsrv_epics_internal_get_callback(
				struct event_handler_args args );

#endif /* __MS_EPICS_H__ */
