/*  dac_test.c  ---  Tests the operation of NI LabPC+ DAC ports.  */

#include <stdio.h>

#include "mx_util.h"
#include "mx_rec.h"
#include "mx_drivr.h"

#include "d_labpc.h"

#define DB_FILE "labpcdac.dat"
#define DB_NEW  "labpcdac.new"

int
main( int argc, char *argv[] )
{
	MX_RECORD *record_list, *dac_record;
	MX_ANALOG_OUTPUT *dac;
	MX_LABPC_DAC *labpc_dac;
	double voltage;
	mx_status_type status;
	int exit_loop;
	char buffer[80];
	char c;

	mx_nap_initialize("junk");

	record_list = mx_initialize_record_list();

	if ( record_list == NULL ) {
		fprintf(stderr,
		"Unable to initialize the MX record list.\n");
		exit(1);
	}

	status = mx_read_database_file( DB_FILE, record_list );

	if ( status.code != MXE_SUCCESS ) {
		fprintf( stderr,
		"Unable to read the database file '%s'\n", DB_FILE );
		exit(1);
	}

	status = mx_initialize_hardware( record_list );

	if ( status.code != MXE_SUCCESS ) {
		fprintf( stderr,
	"Error encountered while trying to initialize the LabPC+.\n");
		exit(1);
	}

	/* Find the DAC in the record list. */

	dac_record = mx_get_record( record_list, "l_dac1" );

	if ( dac_record == NULL ) {
		fprintf( stderr,
		"Can't find a record named 'l_dac1' in the record list.\n");
		exit(1);
	}

	/* Is this really a LabPC+ DAC? */

	if ( ( dac_record->class != MXR_DEVICE )
	  || ( dac_record->type  != MXC_ANALOG_OUTPUT ) ) {

		fprintf( stderr, "class = %ld\n", dac_record->class );
		fprintf( stderr, "type = %ld\n", dac_record->type );
		fprintf( stderr,
	"The record 'labpc_dac' does not have the correct record type.\n");
		exit(1);
	}

	dac = (MX_ANALOG_OUTPUT *) (dac_record->record_data);

	if ( dac == NULL ) {
		fprintf(stderr,
		"The MX_ANALOG_OUTPUT structure pointer was NULL.\n");
		exit(1);
	}

	labpc_dac = (MX_LABPC_DAC *) ( dac->device_data );

	if ( labpc_dac == NULL ) {
		fprintf(stderr,
		"The MX_LABPC_DAC structure pointer was NULL.\n");
		exit(1);
	}

	/* Now loop until we get an exit command. */

	fprintf(stderr, "LabPC+ DAC.  Type 'h' for help.\n");

	exit_loop = 0;

	while (exit_loop == 0) {
		fprintf(stderr, "-> ");
		fflush(stderr);

		fgets( buffer, sizeof buffer, stdin );

		c = buffer[0];

		switch(c) {
		case 'h':
			fprintf(stderr,
			"'h' - help\n"
			"'d' - set DAC to new voltage\n"
			"'s' - show DAC status\n"
			"'t' - show DAC structure\n"
			"'q' - quit\n");
			break;
		case 'd':
			fprintf(stderr,
			"\nEnter new voltage -> ");

			fgets( buffer, sizeof buffer, stdin );

			if ( sscanf(buffer, "%lg", &voltage) != 1 ) {
				fprintf(stderr,
				"  Voltage value unrecognized.  Try again.\n");
			} else {
				status = mx_analog_output_write(
						dac, voltage );

				if ( status.code != MXE_SUCCESS ) {
					fprintf(stderr,
					"Write to DAC failed.\n");
				}
			}
			break;
		case 's':
			status = mx_print_summary( stderr, dac_record );

			if ( status.code != MXE_SUCCESS ) {
				fprintf(stderr, "Show summary failed.\n");
			}
			break;
		case 't':
			status = mx_print_structure( stderr, dac_record );

			if ( status.code != MXE_SUCCESS ) {
				fprintf(stderr, "Show structure failed.\n");
			}
			break;
		case 'q':
			exit_loop = 1;
			break;
		default:
			fprintf(stderr,
			"\nUnrecognized command '%c'\n", c);
			break;
		}
	}

	status = mx_write_database_file(
		DB_NEW, record_list, MXR_INTERFACE | MXR_DEVICE );

	if ( status.code != MXE_SUCCESS ) {
		fprintf(stderr,
		"Attempt to write new database file failed.\n");
		exit(1);
	}

	status = mx_shutdown_hardware( record_list );

	if ( status.code != MXE_SUCCESS ) {
		fprintf(stderr,
		"Attempt to shutdown hardware failed.\n");
		exit(1);
	}

	exit(0);

	return 0;	/* Needed so some compilers won't complain. */
}
