/*
 * Name:    mx_drivr.c
 *
 * Purpose: Describes the list of interfaces and devices included in
 *          this version of the program.
 */

#include <stdio.h>

#include "mx_drivr.h"

/* -- Include header files that define MX_XXX_FUNCTION_LIST structures. -- */

#include "mx_rs232.h"
#include "mx_camac.h"
#include "mx_ain.h"
#include "mx_aout.h"
#include "mx_din.h"
#include "mx_dout.h"
#include "mx_motor.h"
#include "mx_encod.h"
#include "mx_scalr.h"
#include "mx_timer.h"
#include "mx_scan.h"

/* Include the header files for all of the interfaces and devices. */

#include "i_tty.h"
#include "i_com.h"
#include "i_ks3344.h"
#include "i_scamac.h"
#include "i_dsp600.h"
#include "i_pdi45.h"
#include "i_pdi40.h"
#include "d_ks3512.h"
#include "d_ks3112.h"
#include "d_ks3063.h"
#include "d_e500.h"
#include "d_smc24.h"
#include "d_panthr.h"
#include "d_ks3640.h"
#include "d_qs450.h"
#include "d_stimer.h"
#include "d_rtc018.h"
#include "d_pdi45.h"
#include "d_pdi40.h"
#include "d_labpc.h"
#include "s_scaler.h"
#include "s_motor.h"
#include "s_2theta.h"

/* -- Define lists that relate types to classes and to function lists. -- */

MX_TYPE_LIST_ENTRY mx_record_class_list[] = {
{MXR_INTERFACE, "int", 0, NULL, NULL},
{MXR_DEVICE,    "dev", 0, NULL, NULL},
{MXR_SCAN,      "scan", 0, NULL, NULL},
{0, "", 0, NULL, NULL}
};

MX_TYPE_LIST_ENTRY mx_interface_class_list[] = {

  /* First list generic type interfaces. */

{MXI_RS232,   "rs232", MXR_INTERFACE, &mx_rs232_generic_function_list, NULL},
{MXI_IEEE488, "ieee488", MXR_INTERFACE, NULL, NULL},
{MXI_CAMAC,   "camac", MXR_INTERFACE, &mx_camac_generic_function_list, NULL},

  /* Then we follow with multi-io board type interfaces. */

{MXI_PDI45, "pdi45", MXR_INTERFACE, &mxi_pdi45_generic_function_list, NULL},
{MXI_PDI40, "pdi40", MXR_INTERFACE, &mxi_pdi40_generic_function_list, NULL},

{0, "", 0, NULL, NULL}
};


MX_TYPE_LIST_ENTRY mx_interface_type_list[] = {
{MXI_232_SOFTWARE, "rs232_soft", MXI_RS232, NULL, NULL},

#ifdef UNIX
{MXI_232_TTY,      "tty",        MXI_RS232,
		&mxi_tty_generic_function_list,
		&mxi_tty_rs232_function_list},
#endif /* UNIX */

#ifdef MSDOS
{MXI_232_COM,      "com",        MXI_RS232,
		&mxi_com_generic_function_list,
		&mxi_com_rs232_function_list},
#endif /* MSDOS */

{MXI_232_KS3344,   "ks3344",     MXI_RS232,
		&mxi_ks3344_generic_function_list,
		&mxi_ks3344_rs232_function_list},

{MXI_CAM_SOFTWARE, "camac_soft", MXI_CAMAC,
		&mxi_scam_generic_function_list,
		&mxi_scam_camac_function_list},
{MXI_CAM_DSP6001,  "dsp6001",    MXI_CAMAC,
		&mxi_dsp6001_generic_function_list,
		&mxi_dsp6001_camac_function_list},
{0, "", 0, NULL, NULL}
};

MX_TYPE_LIST_ENTRY mx_device_class_list[] = {
{MXC_ANALOG_INPUT,   "analog_input",   MXR_DEVICE,
				&mx_analog_input_generic_function_list,
				NULL},
{MXC_ANALOG_OUTPUT,  "analog_output",  MXR_DEVICE,
				&mx_analog_output_generic_function_list,
				NULL},
{MXC_DIGITAL_INPUT,  "digital_input",  MXR_DEVICE,
				&mx_digital_input_generic_function_list,
				NULL},
{MXC_DIGITAL_OUTPUT, "digital_output", MXR_DEVICE,
				&mx_digital_output_generic_function_list,
				NULL},
{MXC_MOTOR,          "motor",          MXR_DEVICE,
				&mx_motor_generic_function_list,
				NULL},
{MXC_ENCODER,        "encoder",        MXR_DEVICE,
				&mx_encoder_generic_function_list,
				NULL},
{MXC_SCALER,         "scaler",         MXR_DEVICE,
				&mx_scaler_generic_function_list,
				NULL},
{MXC_TIMER,          "timer",          MXR_DEVICE,
				&mx_timer_generic_function_list,
				NULL},
{0, "", 0, NULL, NULL}
};

MX_TYPE_LIST_ENTRY mx_device_type_list[] = {
#if 0
{MXT_AIN_LABPC,         "labpc_adc",     MXC_ANALOG_INPUT,
		&mxd_labpc_adc_generic_function_list,
		&mxd_labpc_adc_analog_input_function_list},
#endif
{MXT_AOU_LABPC,         "labpc_dac",     MXC_ANALOG_OUTPUT,
		&mxd_labpc_dac_generic_function_list,
		&mxd_labpc_dac_analog_output_function_list},
#if 0
{MXT_DIN_LABPC,         "labpc_din",     MXC_DIGITAL_INPUT,
		&mxd_labpc_din_generic_function_list,
		&mxd_labpc_din_digital_input_function_list},
{MXT_DOU_LABPC,         "labpc_dout",    MXC_DIGITAL_OUTPUT,
		&mxd_labpc_dout_generic_function_list,
		&mxd_labpc_dout_digital_output_function_list},
{MXT_SCL_LABPC,         "labpc_scaler",  MXC_SCALER,
		&mxd_labpc_scaler_generic_function_list,
		&mxd_labpc_scaler_scaler_function_list},
{MXT_TIM_LABPC,         "labpc_timer",   MXC_TIMER,
		&mxd_labpc_timer_generic_function_list,
		&mxd_labpc_timer_timer_function_list},
#endif

{0, "", 0, NULL, NULL}
};

MX_TYPE_LIST_ENTRY mx_scan_class_list[] = {
{MXS_SCAN_1D, "scan_1d", MXR_SCAN, NULL, NULL},
{0, "", 0, NULL, NULL}
};

MX_TYPE_LIST_ENTRY mx_scan_type_list[] = {
{MXS_1D_SCALER, "scaler", MXS_SCAN_1D,
		&mxs_scan_scaler_generic_function_list,
		&mxs_scan_scaler_scan_1d_function_list},
{MXS_1D_MOTOR,  "motor",  MXS_SCAN_1D,
		&mxs_scan_motor_generic_function_list,
		&mxs_scan_motor_scan_1d_function_list},
{MXS_1D_2THETA, "2theta", MXS_SCAN_1D,
		&mxs_scan_2theta_generic_function_list,
		&mxs_scan_2theta_scan_1d_function_list},
{0, "", 0, NULL, NULL}
};

