/*
 * Name:    mx_drivr.c
 *
 * Purpose: Describes the list of interfaces and devices included in
 *          this version of the program.
 */

#include <stdio.h>

#include "unistd.h"	/* for Microsoft C 6.0 */

#include "mx_drivr.h"

/* -- Include header files that define MX_XXX_FUNCTION_LIST structures. -- */

#include "mx_rs232.h"
#include "mx_camac.h"
#include "mx_ain.h"
#include "mx_aout.h"
#include "mx_din.h"
#include "mx_dout.h"
#include "mx_motor.h"
#include "mx_scalr.h"
#include "mx_timer.h"
#include "mx_scan.h"

/* Include the header files for all of the interfaces and devices. */

#include "i_tty.h"
#include "i_com.h"
#include "i_scamac.h"
#include "i_dsp600.h"
#include "i_pdi45.h"
#include "d_ks3512.h"
#include "d_ks3112.h"
#include "d_ks3063.h"
#include "d_e500.h"
#include "d_qs450.h"
#include "d_stimer.h"
#include "d_rtc018.h"
#include "s_scaler.h"
#include "s_motor.h"
#include "s_2theta.h"

/* -- Define lists that relate types to classes and to function lists. -- */

MX_TYPE_LIST_ENTRY mx_record_class_list[] = {
{MXR_INTERFACE, "int", 0, NULL, NULL, NULL},
{MXR_DEVICE,    "dev", 0, NULL, NULL, NULL},
{MXR_SCAN,      "scan", 0, NULL, NULL, NULL},
{0, "", 0, NULL, NULL, NULL}
};

MX_TYPE_LIST_ENTRY mx_interface_class_list[] = {

  /* First list generic type interfaces. */

{MXI_RS232, "rs232", MXR_INTERFACE,
		mx_rs232_read_record_description, NULL, NULL},
#if 0
#{MXI_GPIB,  "gpib",  MXR_INTERFACE,
#		NULL, NULL, NULL},
#{MXI_CAMAC, "camac", MXR_INTERFACE,
#		mx_camac_read_record_description, NULL, NULL},
#endif

  /* Then we follow with multi-io board type interfaces. */

{MXI_PDI45, "pdi45", MXR_INTERFACE,
		mxi_pdi45_read_record_description,
		&mxi_pdi45_generic_function_list,
		NULL},

{0, "", 0, NULL, NULL, NULL}
};


MX_TYPE_LIST_ENTRY mx_interface_type_list[] = {
{MXI_232_SOFTWARE, "rs232_soft", MXI_RS232, NULL, NULL, NULL},

#ifdef UNIX
{MXI_232_TTY,      "tty",        MXI_RS232,
		mxi_tty_read_record_description,
		&mxi_tty_generic_function_list,
		&mxi_tty_rs232_function_list},
#endif /* UNIX */

#ifdef MSDOS
{MXI_232_COM,      "com",        MXI_RS232,
		mxi_com_read_record_description,
		&mxi_com_generic_function_list,
		&mxi_com_rs232_function_list},
#endif /* MSDOS */

#if 0
#{MXI_CAM_SOFTWARE, "camac_soft", MXI_CAMAC,
#		mxi_scam_read_record_description,
#		&mxi_scam_generic_function_list,
#		&mxi_scam_camac_function_list},
#{MXI_CAM_DSP6001,  "dsp6001",    MXI_CAMAC,
#		mxi_dsp6001_read_record_description,
#		&mxi_dsp6001_generic_function_list,
#		&mxi_dsp6001_camac_function_list},
#endif

{0, "", 0, NULL, NULL, NULL}
};

MX_TYPE_LIST_ENTRY mx_device_class_list[] = {
{MXC_ANALOG_INPUT,   "analog_input",   MXR_DEVICE,
		mx_analog_input_read_record_description, NULL, NULL},
{MXC_ANALOG_OUTPUT,  "analog_output",  MXR_DEVICE,
		mx_analog_output_read_record_description, NULL, NULL},
{MXC_DIGITAL_INPUT,  "digital_input",  MXR_DEVICE,
		mx_digital_input_read_record_description, NULL, NULL},
{MXC_DIGITAL_OUTPUT, "digital_output", MXR_DEVICE,
		mx_digital_output_read_record_description, NULL, NULL},
{MXC_MOTOR,          "motor",          MXR_DEVICE,
		mx_motor_read_record_description, NULL, NULL},
{MXC_ENCODER,        "encoder",        MXR_DEVICE, NULL, NULL, NULL},
{MXC_SCALER,         "scaler",         MXR_DEVICE,
		mx_scaler_read_record_description, NULL, NULL},
{MXC_TIMER,          "timer",          MXR_DEVICE,
		mx_timer_read_record_description, NULL, NULL},
{0, "", 0, NULL, NULL, NULL}
};

MX_TYPE_LIST_ENTRY mx_device_type_list[] = {
#if 0
#{MXT_AIN_KS3512,   "ks3512",     MXC_ANALOG_INPUT,
#		mxd_ks3512_read_record_description,
#		&mxd_ks3512_generic_function_list,
#		&mxd_ks3512_analog_input_function_list},
#{MXT_AOU_KS3112,   "ks3112",     MXC_ANALOG_OUTPUT,
#		mxd_ks3112_read_record_description,
#		&mxd_ks3112_generic_function_list,
#		&mxd_ks3112_analog_output_function_list},
#{MXT_DIN_KS3063,   "ks3063_in",  MXC_DIGITAL_INPUT,
#		mxd_ks3063_in_read_record_description,
#		&mxd_ks3063_in_generic_function_list,
#		&mxd_ks3063_in_digital_input_function_list},
#{MXT_DOU_KS3063,   "ks3063_out", MXC_DIGITAL_OUTPUT,
#		mxd_ks3063_out_read_record_description,
#		&mxd_ks3063_out_generic_function_list,
#		&mxd_ks3063_out_digital_output_function_list},
#{MXT_MTR_SOFTWARE, "motor_soft", MXC_MOTOR, NULL, NULL, NULL},
#{MXT_MTR_E500,     "e500",       MXC_MOTOR,
#		mxd_e500_read_record_description,
#		&mxd_e500_generic_function_list,
#		&mxd_e500_motor_function_list},
#{MXT_MTR_SMC24,    "smc24",      MXC_MOTOR, NULL, NULL, NULL},
#{MXT_SCL_QS450,    "qs450",      MXC_SCALER,
#		mxd_qs450_read_record_description,
#		&mxd_qs450_generic_function_list,
#		&mxd_qs450_scaler_function_list},
#{MXT_SCL_KS3610,   "ks3610",     MXC_SCALER,
#		mxd_qs450_read_record_description,
#		&mxd_qs450_generic_function_list,
#		&mxd_qs450_scaler_function_list},
#{MXT_TIM_SOFTWARE, "stimer",     MXC_TIMER,
#		mxd_stimer_read_record_description,
#		&mxd_stimer_generic_function_list,
#		&mxd_stimer_timer_function_list},
#{MXT_TIM_RTC018,   "rtc018",     MXC_TIMER,
#		mxd_rtc018_read_record_description,
#		&mxd_rtc018_generic_function_list,
#		&mxd_rtc018_timer_function_list},
#endif
{0, "", 0, NULL, NULL, NULL}
};

MX_TYPE_LIST_ENTRY mx_scan_class_list[] = {
{MXS_SCAN_1D, "scan_1d", MXR_SCAN,
		mx_scan_1d_read_record_description, NULL, NULL},
{0, "", 0, NULL, NULL, NULL}
};

MX_TYPE_LIST_ENTRY mx_scan_type_list[] = {
{MXS_1D_SCALER, "scaler", MXS_SCAN_1D,
		mxs_scan_scaler_read_record_description,
		&mxs_scan_scaler_generic_function_list,
		&mxs_scan_scaler_scan_1d_function_list},
{MXS_1D_MOTOR,  "motor",  MXS_SCAN_1D,
		mxs_scan_motor_read_record_description,
		&mxs_scan_motor_generic_function_list,
		&mxs_scan_motor_scan_1d_function_list},
{MXS_1D_2THETA, "2theta", MXS_SCAN_1D,
		mxs_scan_2theta_read_record_description,
		&mxs_scan_2theta_generic_function_list,
		&mxs_scan_2theta_scan_1d_function_list},
{0, "", 0, NULL, NULL, NULL}
};

#ifndef OS_LINUX

unsigned int
sleep( unsigned int seconds )
{
	/* Kludge alert! Do nothing for now. This is only here to allow
	 * the program to link.  We need to rewrite mx_nap.c.
	 */

	return 0;
}

#endif /* OS_LINUX */
