
#include <stdio.h>

#include "mx_util.h"
#include "mx_rec.h"
#include "mx_drivr.h"
#include "pdi45.h"

int
main( int argc, char *argv[] )
{
	MX_RECORD *record_list, *record;
	MX_PDI45 *pdi45;
	double voltage;
	mx_status_type status;

	mx_nap_initialize("junk");

	record_list = mx_initialize_record_list();

	if ( record_list == NULL ) {
		fprintf(stderr,
		"Unable to initialize the MX record list.\n");
		exit(1);
	}

	status = mx_read_database_file( "pdi45.dat", record_list );

	if ( status != SUCCESS ) {
		fprintf(stderr,
		"Unable to read the database file 'pdi45.dat'\n");
		exit(1);
	}

	status = mx_initialize_hardware( record_list );

	if ( status != SUCCESS ) {
		fprintf(stderr,
"Error encountered while trying to initialize the data acquisition hardware.\n");
		exit(1);
	}

	/* Find the PDI 45 in the record list. */

	record = mx_get_record( record_list, "pdi45_1" );

	if ( record == NULL ) {
		fprintf(stderr,
		"Can't find a record named 'pdi45_1' in the record list.\n");
		exit(1);
	}

	/* Is this a PDI45? */

	if ( (record->class != MXR_INTERFACE)
	  || (record->type  != MXI_PDI45) ) {
		fprintf(stderr,
		"The record 'pdi45' does not have the correct record type.\n");
		exit(1);
	}

	pdi45 = (MX_PDI45 *) (record->record_data);

	if ( pdi45 == NULL ) {
		fprintf(stderr,
		"The MX_PDI45 structure pointer was NULL.\n");
		exit(1);
	}

	/* Now loop forever. */

	while(1) {
		/* Read the input voltage. */

		status = pdi45_read_analog_input( pdi45, 0, &voltage );

		fprintf(stderr,"\nV = %f volts\n\n", voltage);

		/* Sleep for 1 second. */

		mx_nap( 5000 );
	}

	/* We shouldn't be able to get here. */

	return 1;
}
